@@ -801,8 +801,10 @@ gs_plugin_fwupd_install (GsPlugin *plugin,
 		gs_app_set_state_recover (app);
 		return FALSE;
 	}
+
+	/* delete the file from the cache */
 	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
-	return TRUE;
+	return g_file_delete (local_file, cancellable, error);
 }
 
 static gboolean
@@ -1012,7 +1014,7 @@ gs_plugin_add_sources (GsPlugin *plugin,
 		g_autoptr(GsApp) app = NULL;
 
 		/* ignore these, they're built in */
-		if (fwupd_remote_get_kind (remote) == FWUPD_REMOTE_KIND_LOCAL)
+		if (fwupd_remote_get_kind (remote) != FWUPD_REMOTE_KIND_DOWNLOAD)
 			continue;
 
 		/* create something that we can use to enable/disable */
