@@ -62,7 +62,7 @@ gs_loading_page_status_changed_cb (GsPluginLoader *plugin_loader,
 		}
 	} else {
 		/* TRANSLATORS: initial start */
-		str = _("Software catalog is being loaded");
+		str = _("Software catalog is being downloaded");
 	}
 
 	/* update label */
