@@ -45,7 +45,6 @@ static const gchar *page_name[] = {
 typedef struct {
 	GsShellMode	 mode;
 	GtkWidget	*focus;
-	GsApp		*app;
 	GsCategory	*category;
 	gchar		*search;
 } BackEntry;
@@ -195,7 +194,6 @@ free_back_entry (BackEntry *entry)
 		g_object_remove_weak_pointer (G_OBJECT (entry->focus),
 		                              (gpointer *) &entry->focus);
 	g_clear_object (&entry->category);
-	g_clear_object (&entry->app);
 	g_free (entry->search);
 	g_free (entry);
 }
@@ -383,12 +381,6 @@ save_back_entry (GsShell *shell)
 	BackEntry *entry;
 	GsPage *page;
 
-	/* never go back to a details page */
-	if (priv->mode == GS_SHELL_MODE_DETAILS) {
-		g_debug ("ignoring back entry for details");
-		return;
-	}
-
 	entry = g_new0 (BackEntry, 1);
 	entry->mode = priv->mode;
 
@@ -485,11 +477,6 @@ gs_shell_go_back (GsShell *shell)
 			 gs_category_get_id (entry->category));
 		gs_shell_change_mode (shell, entry->mode, entry->category, FALSE);
 		break;
-	case GS_SHELL_MODE_DETAILS:
-		g_debug ("popping back entry for %s with %p",
-			 page_name[entry->mode], entry->app);
-		gs_shell_change_mode (shell, entry->mode, entry->app, FALSE);
-		break;
 	case GS_SHELL_MODE_SEARCH:
 		g_debug ("popping back entry for %s with %s",
 			 page_name[entry->mode], entry->search);
@@ -535,24 +522,47 @@ gs_shell_reload_cb (GsPluginLoader *plugin_loader, GsShell *shell)
 }
 
 static void
-initial_refresh_done (GsLoadingPage *loading_page, gpointer data)
+overview_page_refresh_done (GsOverviewPage *overview_page, gpointer data)
 {
-	GsPage *page;
 	GsShell *shell = data;
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+	GsPage *page;
 
-	g_signal_handlers_disconnect_by_func (loading_page, initial_refresh_done, data);
+	g_signal_handlers_disconnect_by_func (overview_page, overview_page_refresh_done, data);
 
 	page = GS_PAGE (gtk_builder_get_object (priv->builder, "updates_page"));
 	gs_page_reload (page);
 	page = GS_PAGE (gtk_builder_get_object (priv->builder, "installed_page"));
 	gs_page_reload (page);
 
+	gs_shell_change_mode (shell, GS_SHELL_MODE_OVERVIEW, NULL, TRUE);
+
+	/* now that we're finished with the loading page, connect the reload signal handler */
+	g_signal_connect (priv->plugin_loader, "reload",
+	                  G_CALLBACK (gs_shell_reload_cb), shell);
+}
+
+static void
+initial_refresh_done (GsLoadingPage *loading_page, gpointer data)
+{
+	GsShell *shell = data;
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+
+	g_signal_handlers_disconnect_by_func (loading_page, initial_refresh_done, data);
+
 	g_signal_emit (shell, signals[SIGNAL_LOADED], 0);
 
-	/* go to OVERVIEW, unless the "loading" callbacks changed mode already */
-	if (priv->mode == GS_SHELL_MODE_LOADING)
-		gs_shell_change_mode (shell, GS_SHELL_MODE_OVERVIEW, NULL, TRUE);
+	/* if the "loaded" signal handler didn't change the mode, kick off async
+	 * overview page refresh, and switch to the page once done */
+	if (priv->mode == GS_SHELL_MODE_LOADING) {
+		GsPage *page;
+
+		page = GS_PAGE (gtk_builder_get_object (priv->builder, "overview_page"));
+		g_signal_connect (page, "refreshed",
+		                  G_CALLBACK (overview_page_refresh_done), shell);
+		gs_page_reload (page);
+		return;
+	}
 
 	/* now that we're finished with the loading page, connect the reload signal handler */
 	g_signal_connect (priv->plugin_loader, "reload",
@@ -2132,6 +2142,19 @@ gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *can
 	gs_shell_change_mode (shell, GS_SHELL_MODE_LOADING, NULL, TRUE);
 }
 
+void
+gs_shell_reset_state (GsShell *shell)
+{
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+
+	/* reset to overview, unless we're in the loading state which advances
+	 * to overview on its own */
+	if (priv->mode != GS_SHELL_MODE_LOADING)
+		priv->mode = GS_SHELL_MODE_OVERVIEW;
+
+	gs_shell_clean_back_entry_stack (shell);
+}
+
 void
 gs_shell_set_mode (GsShell *shell, GsShellMode mode)
 {
